import { ref, watch } from "vue";
import { PagodaUiError } from "../../../utils/error.mjs";
import { isBoolean } from "lodash-unified";
function useTabs({
  activeTab,
  defaultTabs = [],
  beforeTabsRemove,
  onActiveTabChange
}) {
  const currentTabName = ref("");
  const tabs = ref([...defaultTabs]);
  watch(
    activeTab,
    (activeTabConfig) => {
      if (activeTabConfig) {
        if (!activeTabConfig.name) {
          throw new PagodaUiError(
            "Component Layout",
            "activeTab \u5C5E\u6027\u7F3A\u5C11\u552F\u4E00\u6807\u8BC6 name \u5B57\u6BB5"
          );
        }
        const tabIndex = tabs.value.findIndex(
          (tab) => tab.name === activeTabConfig.name
        );
        const newTabConfig = Object.assign({}, activeTabConfig);
        if (tabIndex === -1) {
          tabs.value.push(newTabConfig);
        } else {
          tabs.value[tabIndex] = newTabConfig;
        }
        currentTabName.value = activeTabConfig.name;
      } else {
        currentTabName.value = "";
      }
    },
    {
      immediate: true
    }
  );
  function isTabClosable(tab) {
    return "closable" in tab && isBoolean(tab.closable) ? tab.closable : true;
  }
  function hasTab(tabName) {
    return tabs.value.some((tab) => tab.name === tabName);
  }
  function selectTab(tab) {
    const tabName = tab.props.name;
    if (tabName !== currentTabName.value) {
      const selectedTab = tabs.value.find(
        (tabConfig) => tabConfig.name === tabName
      );
      if (selectedTab)
        onActiveTabChange == null ? void 0 : onActiveTabChange(selectedTab);
    }
  }
  function closeActiveTab() {
    if (activeTab.value) {
      closeTab(activeTab.value.name);
    }
  }
  function closeTab(removeTabName, safeClose) {
    const isTabShouldClose = (tab) => isTabClosable(tab) && tab.name === removeTabName;
    const next = () => {
      tabs.value = tabs.value.filter((tab, index) => {
        if (activeTab.value && activeTab.value.name === removeTabName && tab.name === removeTabName && !safeClose) {
          const nextActiveTab = tabs.value[index + 1] || tabs.value[index - 1];
          if (nextActiveTab) {
            onActiveTabChange == null ? void 0 : onActiveTabChange(nextActiveTab);
          }
        }
        return !isTabShouldClose(tab);
      });
    };
    if (beforeTabsRemove) {
      const removeTabConfig = tabs.value.find(
        (tab) => tab.name === removeTabName
      );
      if (removeTabConfig)
        beforeTabsRemove([removeTabConfig], next, safeClose);
    } else {
      next();
    }
  }
  function closeAllTabs(safeClose) {
    const isTabShouldClose = (tab) => isTabClosable(tab);
    const next = () => {
      tabs.value = tabs.value.filter((tab) => {
        return !isTabShouldClose(tab);
      });
      if (tabs.value.length && !safeClose && activeTab.value && !hasTab(activeTab.value.name)) {
        onActiveTabChange == null ? void 0 : onActiveTabChange(tabs.value[tabs.value.length - 1]);
      }
    };
    if (beforeTabsRemove) {
      const canCloseAllTabs = tabs.value.filter((tab) => {
        return isTabShouldClose(tab);
      });
      beforeTabsRemove(canCloseAllTabs, next, safeClose);
    } else {
      next();
    }
  }
  function closeOthersTabs(ignoreTabName, safeClose) {
    const isTabShouldClose = (tab) => isTabClosable(tab) && tab.name !== ignoreTabName;
    const next = () => {
      tabs.value = tabs.value.filter((tab) => {
        return !isTabShouldClose(tab);
      });
      if (tabs.value.length && !safeClose && activeTab.value && !hasTab(activeTab.value.name)) {
        onActiveTabChange == null ? void 0 : onActiveTabChange(tabs.value[tabs.value.length - 1]);
      }
    };
    if (beforeTabsRemove) {
      const othersCanCloseTabs = tabs.value.filter((tab) => {
        return isTabShouldClose(tab);
      });
      beforeTabsRemove(othersCanCloseTabs, next, safeClose);
    } else {
      next();
    }
  }
  return {
    currentTabName,
    tabs,
    isTabClosable,
    selectTab,
    closeActiveTab,
    closeTab,
    closeAllTabs,
    closeOthersTabs
  };
}
export {
  useTabs
};
