import { reactive } from "vue";
import { cloneDeep } from "lodash-unified";
import { loopTree } from "pagoda-shared";
function useTableStore(options) {
  const {
    data,
    getColumns = () => [],
    rowStatesCreator,
    getCellsStatesCreator,
    childrenKey = "children"
  } = options;
  const store = {
    states: /* @__PURE__ */ new WeakMap(),
    tableCacheData: /* @__PURE__ */ new WeakMap()
  };
  if (data) {
    addStore(data);
  }
  function resetStore(data2) {
    store.states = /* @__PURE__ */ new WeakMap();
    store.tableCacheData = /* @__PURE__ */ new WeakMap();
    addStore(data2);
  }
  function getRowsCache(rows) {
    return rows.map((row) => {
      return store.tableCacheData.get(row);
    });
  }
  function setRowsCache(rows) {
    rows.forEach((row) => {
      store.tableCacheData.set(row, cloneDeep(row));
    });
  }
  function deleteRowsCache(rows) {
    rows.forEach((row) => {
      store.tableCacheData.delete(row);
    });
  }
  function addStore(data2, parent) {
    loopTree(
      data2,
      childrenKey,
      (row, parent2) => {
        const rowConfig = {
          states: {},
          cellsStates: {}
        };
        if (rowStatesCreator) {
          rowConfig.states = reactive(rowStatesCreator(row, parent2));
        }
        const cellsStatesCreator = getCellsStatesCreator == null ? void 0 : getCellsStatesCreator();
        if (cellsStatesCreator) {
          getColumns().forEach((prop) => {
            const cellStatesCreator = cellsStatesCreator[prop];
            if (cellStatesCreator) {
              rowConfig.cellsStates[prop] = reactive(
                cellStatesCreator(row, parent2)
              );
            }
          });
        }
        store.states.set(row, rowConfig);
        setRowsCache([row]);
      },
      parent
    );
  }
  function resetCellsStates(data2) {
    if (!getCellsStatesCreator)
      return;
    const columns = getColumns();
    const cellsStatesCreator = getCellsStatesCreator();
    loopTree(data2, childrenKey, (row, parent) => {
      const rowConfig = store.states.get(row);
      if (!rowConfig)
        return;
      Object.keys(rowConfig.cellsStates).forEach((prop) => {
        if (!columns.includes(prop)) {
          delete rowConfig.cellsStates[prop];
        }
      });
      columns.forEach((prop) => {
        const cellStatesCreator = cellsStatesCreator[prop];
        if (cellStatesCreator && !rowConfig.cellsStates[prop]) {
          rowConfig.cellsStates[prop] = reactive(
            cellStatesCreator(row, parent)
          );
        }
      });
    });
  }
  function getRowStates(row) {
    var _a;
    return (_a = store.states.get(row)) == null ? void 0 : _a.states;
  }
  function getCellsStates(row) {
    var _a;
    return (_a = store.states.get(row)) == null ? void 0 : _a.cellsStates;
  }
  function getCellStates(row, cell) {
    var _a, _b;
    return (_b = (_a = store.states.get(row)) == null ? void 0 : _a.cellsStates) == null ? void 0 : _b[cell];
  }
  function setRowsStates(rows, states) {
    loopTree(rows, childrenKey, (row) => {
      const rowStates = getRowStates(row);
      if (rowStates) {
        Object.assign(rowStates, states);
      }
    });
  }
  function setCellStates(rows, cells, states) {
    loopTree(rows, childrenKey, (row) => {
      const cellsStates = getCellsStates(row);
      cells.forEach((cell) => {
        if (cellsStates == null ? void 0 : cellsStates[cell]) {
          Object.assign(cellsStates[cell], states);
        }
      });
    });
  }
  function delRowsStates(rows) {
    loopTree(rows, childrenKey, (row) => {
      store.states.delete(row);
    });
  }
  function delRowsStore(rows) {
    loopTree(rows, childrenKey, (row) => {
      store.states.delete(row);
      store.tableCacheData.delete(row);
    });
  }
  return Object.assign(store, {
    addStore,
    resetStore,
    setRowsStates,
    setCellStates,
    resetCellsStates,
    setRowsCache,
    getRowsCache,
    getCellStates,
    getRowStates,
    getCellsStates,
    delRowsStates,
    deleteRowsCache,
    delRowsStore
  });
}
export {
  useTableStore
};
